import { __decorate } from "tslib";
import { Datasource } from '@genesislcap/foundation-comms';
import { NotificationEvent, showNotification, getNotificationBuilder, getToastBuilder, } from '@genesislcap/foundation-notifications';
import { createLogger } from '@genesislcap/foundation-utils';
import { attr, nullableNumberConverter, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { foundationNotificationListenerTemplate as template } from './notification-listener.template';
export const foundationNotificationListenerShadowOptions = {
    mode: 'open',
    delegatesFocus: true,
};
/**
 * @tagname %%prefix%%-notification-listener
 */
export class NotificationListener extends FoundationElement {
    constructor() {
        super(...arguments);
        this.logger = createLogger('notification-listener');
        /**
         * Number of milliseconds before toast will disappear. Leave empty or set to 0 to keep toast open indefinitely
         * @public
         */
        this.closeTimeout = 0;
        /**
         * Array of buttons for notification toasts
         * @public
         */
        this.toastButtons = [];
    }
    async connectedCallback() {
        super.connectedCallback();
        this.addEventListener(NotificationEvent.NOTIFICATION_EVENT, (e) => {
            e.stopImmediatePropagation();
            showNotification(e.detail, this.tagName, this.notificationListener);
        });
        if (this.resourceName) {
            await this.datasource.init({
                resourceName: this.resourceName,
            }, true);
            this.datasource.stream?.subscribe((result) => {
                if (!result.ROW) {
                    return;
                }
                const details = result?.ROW[0];
                if (result.SEQUENCE_ID > 1 && details.DETAILS.OPERATION === 'INSERT') {
                    const title = details.HEADER;
                    const message = details.MESSAGE;
                    const type = details.NOTIFY_SEVERITY?.toLowerCase();
                    const createdAt = details.CREATED_AT ? new Date(details.CREATED_AT) : undefined;
                    const expiry = details.EXPIRY ? details.EXPIRY - details.CREATED_AT : 0;
                    let buttons = [];
                    if (this.toastButtons && this.toastButtons.length > 0) {
                        buttons = this.toastButtons.filter(({ condition }) => !!condition ? condition({ details }) : true);
                    }
                    showNotification(getNotificationBuilder()
                        .withTitle(title)
                        .withBody(message)
                        .withCreatedAt(createdAt)
                        .withToast(getToastBuilder()
                        .withDetails(details)
                        .withType(type)
                        .withButtons(buttons)
                        .withAutoClose(!!expiry ?? !!this.closeTimeout)
                        .withCloseTimeout(expiry ?? this.closeTimeout)
                        .build())
                        .build(), this.tagName, this.notificationListener);
                }
            });
        }
    }
}
__decorate([
    observable
], NotificationListener.prototype, "notificationListener", void 0);
__decorate([
    attr({ attribute: 'resource-name' })
], NotificationListener.prototype, "resourceName", void 0);
__decorate([
    attr({ attribute: 'close-timeout', converter: nullableNumberConverter })
], NotificationListener.prototype, "closeTimeout", void 0);
__decorate([
    observable
], NotificationListener.prototype, "toastButtons", void 0);
__decorate([
    Datasource
], NotificationListener.prototype, "datasource", void 0);
/**
 * The Foundation Notification Listener
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-notification-listener\>
 */
export const foundationNotificationListener = NotificationListener.compose({
    baseName: 'notification-listener',
    template,
    shadowOptions: foundationNotificationListenerShadowOptions,
});
